@@ -21,7 +21,7 @@ extern "C" {
 #endif
 
 // *** GENERATED FILE (coords.cpp), DO NOT CHANGE OR PRETTIFY ***
-static const char *TERRITORY_LOCAL_NAME_UTF8[] = {
+static const char *TERRITORY_FULL_NAME_LOCAL[] = {
         "Sancta Sedes|Holy See",
         "Monaco",
         "Gibraltar",
@@ -554,8 +554,8 @@ static const char *TERRITORY_LOCAL_NAME_UTF8[] = {
         "United States Minor Outlying Islands|USMOI",
         "Île de Clipperton|Isla de la Pasión",
         "Antarctica",
-        "Earth",
-        "?"};
+        "Earth"
+};
 
 #ifdef __cplusplus
 }
